<?php
/*
 * the home page
 */
require_once("include.php");
require_once("postgresql.php");
InputPage::$method = "post";

class ButtonAskDeleteServer extends InputPageButtonOnlyAction {
    public function ButtonAskDeleteServer($name, $value) {
        parent::__construct($name, $value);
        $this->anyway = true;
    }
    
    function action(InputPage &$page, $phase) {
        $page->warp($_SERVER["SCRIPT_NAME"]."?_page=".ExchangeConfigConsts::pageConfig."&".$page->getArgs()."&mode=".ExchangeConfigConsts::modeDeleteServer);
    }
}

class ButtonDeleteServer extends MainInputPageOkCancelButton {
    public function ButtonDeleteServer($name, $pageNo, InputPage &$page, $nextMode, $prevMode) {
        parent::__construct($name, $pageNo, $page, $nextMode, $prevMode);
    }
    
    function action(InputPage &$page, $phase) {
        /* @var $exchangeSession ExchangeSession*/global $exchangeSession;
        if($exchangeSession->exServer->delete() === false) {
            return $this->reRenderPage($page, $exchangeSession->exServer->error);
        }
        $page->warp($_SERVER["SCRIPT_NAME"]."?_page=".ExchangeConfigConsts::pageConfig."&tab=".ExchangeConfigConsts::tabServer."&server_id=0&deletedserver=".urlencode($page->serverName));
    }
    
    function reRenderPage(&$page, $error) {
        $this->msg = $error;
        $page->pages[$page->pageNo]->fillLists();
        $page->render();
        return false;
    }
}

class ButtonTestPbx extends InputPageButtonOnlyAction {
    public function ButtonTestPbx($name, $value) {
        parent::__construct($name, $value);
        $this->anyway = false;
    }
    
    function action(InputPage &$page, $phase) {
        /* @var $exchangeSession ExchangeSession*/global $exchangeSession;
        if(ButtonPbxConfig::setValues($page) === false) {
            $page->render();
            return;
        }
        /* @var $pbx pbx */ $pbx = $exchangeSession->exServer->pbxSoap();
        if($pbx === false) {
            $this->msg = $exchangeSession->exServer->error;
        }
        else {
            try
            {
                $pbx->_Echo();
                $this->msg = tl::tl("Ok")."!";
                $pbx->End();
            }
            catch(Exception $e) {
                $this->msg = $e->getMessage();
            }
        }
        $page->render();
        $this->msg = "";
    }
}

class ButtonTestEws extends InputPageButtonOnlyAction {
    public function ButtonTestEws($name, $value) {
        parent::__construct($name, $value);
        $this->anyway = false;
    }
    
    function action(InputPage &$page, $phase) {
        /* @var $exchangeSession ExchangeSession*/global $exchangeSession;
        if(ButtonExchangeConfig::setValues($page) === false) {
            $page->render();
            return;
        }
        $ews = $exchangeSession->exServer->ewsSoap();
        if($ews === false) {
            $this->msg = $exchangeSession->exServer->error;
        }
        else {
            try
            {
                $version = new EWS_RequestServerVersion();
                $version->Version = $exchangeSession->exServer->ews->version;
                $request = new EWS_GetItemType();
                $request->ItemShape = new EWS_ItemResponseShapeType();
                $request->ItemShape->BaseShape = EWS_ItemResponseShapeType::BaseShape_AllProperties;
                $calId = new EWS_ItemIdType();
                $calId->Id = "AAMkADk";
                $request->ItemIds = new EWS_NonEmptyArrayOfBaseItemIdsType();
                $request->ItemIds->ItemId[] = $calId;
                $response = $ews->GetItem($request, null, null, $version);
                $this->msg = tl::tl("Ok")."!";
            }
            catch(Exception $e) {
                $this->msg = $e->getMessage();
            }
        }
        $page->render();
        $this->msg = "";
    }
}

class ButtonServerConfig extends MainInputPageOkCancelButton {
    public function ButtonServerConfig($name, $pageNo, InputPage &$page, $nextMode = null, $prevMode = null) {
        parent::__construct($name, $pageNo, $page, $nextMode, $prevMode);
        $this->anyway = false;
    }

    function action(InputPage &$page, $phase) {
        $name = $page->getField("name");
        $desc = $page->getField("desc");
        $trace = $page->getField("trace");
        $lang = $page->getField("calendarlang");
        $interval = $page->getField("interval");
        
        /* @var $exchangeSession ExchangeSession*/global $exchangeSession;
        $exchangeSession->exServer->name = $name->value;
        $exchangeSession->exServer->desc = $desc->value;
        $exchangeSession->exServer->lang = array_search(true, $lang->value);
        $exchangeSession->exServer->trace = ($trace->value["on"] ? true : false);
        $exchangeSession->exServer->interval = $interval->value;
        if($exchangeSession->exServer->checkName($name->value) === false) {
            return $this->error($page, $exchangeSession->exServer->error);
        }
        $exchangeSession->updateExchangeServer($exchangeSession->exServer);
        
        $page->warp($_SERVER["SCRIPT_NAME"]."?_page=".ExchangeConfigConsts::pageConfig."&".$page->getArgs()."&mode=".$this->nextMode);
    }

    function error(InputPage &$page, $error) {
        $this->msg = $error;
        $page->render();
        return false;
    }
}

class ButtonExchangeConfig extends MainInputPageOkCancelButton {
    public function ButtonExchangeConfig($name, $pageNo, InputPage &$page, $nextMode = null, $prevMode = null) {
        parent::__construct($name, $pageNo, $page, $nextMode, $prevMode, null, true);
        $this->anyway = false;
    }

    function action(InputPage &$page, $phase) {
        if(self::setValues($page) === false) {
            return $this->error($page, "");
        }
        if($this->prevButton != null) {
            /* @var $exchangeSession ExchangeSession*/global $exchangeSession;
            if($exchangeSession->exServer->name == null || $exchangeSession->exServer->name == "") {
                home_die(tl::tl("Session exceeded!"), "config_exchange.php");
            }
            $mode = ($exchangeSession->exServer->id == 0 ? "createdserver" : "updatedserver");
            if($exchangeSession->exServer->write() === false) {
                return $this->error($page, $exchangeSession->exServer->error);
            }
            $arg = "";
            if($page->serverId === 0) {
                $arg = "&createdserver=".urlencode($exchangeSession->exServer->name);
            }
            else {
                $arg = "&editedserver=".urlencode($exchangeSession->exServer->name);
            }
            $serverId = $page->getField("server_id");
            $serverId->value = $page->serverId = 0;
            $page->warp($_SERVER["SCRIPT_NAME"]."?_page=".ExchangeConfigConsts::pageConfig."&".$page->getArgs().$arg);
        }
        else {
            $page->warp($_SERVER["SCRIPT_NAME"]."?_page=".ExchangeConfigConsts::pageConfig."&".$page->getArgs()."&mode={$this->prevMode}");
        }
    }

    static function setValues(InputPage &$page) {
        $server = $page->getField("ewsserver");
        $account = $page->getField("ewsaccount");
        $pw = $page->getField("ewspw");
        $availabilityTimespan = $page->getField("availabilitytimespan");
        $statusFrequency = $page->getField("statusfrequency");
        $tentativeMails = $page->getField("tentativemails");
        $version = $page->getField("version");
        $linuxIp = $page->getField("linuxip");
        $linuxPort = $page->getField("linuxport");
        
        if(!$server->valid() || !$account->valid() || !$pw->valid() || !$availabilityTimespan->valid() || !$statusFrequency->valid() ||
            ($tentativeMails->value != "" && !$tentativeMails->valid()) || !$version->valid() ||
            ($linuxIp->value != "" && !$linuxIp->valid()) ||
            ($linuxPort->value != "" && !$linuxPort->valid())) {
            return false;
        }
        
        /* @var $exchangeSession ExchangeSession*/global $exchangeSession;
        $exchangeSession->exServer->ews->account = $account->value;
        $exchangeSession->exServer->ews->server = $server->value;
        $exchangeSession->exServer->ews->pw = $pw->value;
        $exchangeSession->exServer->ews->statusFrequency = $statusFrequency->value;
        $exchangeSession->exServer->ews->availabilityTimespan = $availabilityTimespan->value;
        $exchangeSession->exServer->ews->tentativeMails = $tentativeMails->value;
        $exchangeSession->exServer->ews->version = array_search("true", $version->value);
        $exchangeSession->exServer->ews->linuxIp = $linuxIp->value;
        $exchangeSession->exServer->ews->linuxPort = $linuxPort->value;
        $exchangeSession->updateExchangeServer($exchangeSession->exServer);
        return true;
    }
    
    function error(InputPage &$page, $error) {
        $this->msg = $error;
        $page->render();
        return false;
    }
}
    
class ButtonPbxConfig extends MainInputPageOkCancelButton {
    public function ButtonPbxConfig($name, $pageNo, InputPage &$page, $nextMode = null, $prevMode = null) {
        parent::__construct($name, $pageNo, $page, $nextMode, $prevMode, null, true);
        $this->anyway = false;
    }

    function action(InputPage &$page, $phase) {
        if(self::setValues($page) === false) {
            return $this->error($page, "");
        }
        //$cron = new CronJobs(Config::cronExchange);
        //$exchangeJob = new CronJob();
        //$exchangeJob->min = "*/{$interval->value}";
        //$script = "/var/www/innovaphone/apps/innovaphone-exchange/exchange.php";
        //$exchangeJob->cmd = "root [ -x /usr/bin/php5 ] && nice -1 /usr/bin/php5 -q $script >> ".Config::exchangeLog." 2>&1";
        //$cron->add($script, $exchangeJob);
        //if($cron->submit(Config::cronExchange) === false) {
        //    return $this->error($page, $cron->error);
        //}
        $mode = "";
        if($this->prevButton != null) {
            $mode = $this->nextMode;
        }
        else {
            $mode = $this->prevMode;
        }
        $page->warp($_SERVER["SCRIPT_NAME"]."?_page=".ExchangeConfigConsts::pageConfig."&".$page->getArgs()."&mode=$mode");
    }
    
    static function setValues(InputPage &$page) {
        $ip = $page->getField("ip");
        $account = $page->getField("account");
        $pw = $page->getField("pw");
        $dynPbx = $page->getField("dynpbx");
        $ssl = $page->getField("ssl");
        $port = $page->getField("port");
        
        if(!$ip->valid() || !$account->valid() || !$pw->valid() || ($port->value != "" && !$port->valid())) {
            return false;
        }
        
        /* @var $exchangeSession ExchangeSession*/global $exchangeSession;
        $exchangeSession->exServer->pbx->account = $account->value;
        $exchangeSession->exServer->pbx->ip = $ip->value;
        $exchangeSession->exServer->pbx->pw = $pw->value;
        $exchangeSession->exServer->pbx->dynPbx = $dynPbx->value;
        $exchangeSession->exServer->pbx->ssl = $ssl->value["on"] ? true : false;
        $exchangeSession->exServer->pbx->port = $port->value;
        $exchangeSession->updateExchangeServer($exchangeSession->exServer);
        return true;
    }

    function error(InputPage &$page, $error) {
        $this->msg = $error;
        $page->render();
        return false;
    }
}

class PageConfig extends PostgresPage {
    public function getHelpAnchor() {
        switch($this->parent->currentTab) {
            case ExchangeConfigConsts::tabPgsql: 
                switch($this->parent->currentMode) {
                    case PostgresPage::modeInRePgPw:
                        return "#Password";
                    default: return "#Database";
                }
            case ExchangeConfigConsts::tabServer: 
                switch($this->parent->currentMode) {
                    case ExchangeConfigConsts::modeServer: 
                        return "#Server";
                    case ExchangeConfigConsts::modeEws: 
                        return "#Exchange";
                    case ExchangeConfigConsts::modePbx: 
                        return "#PBX";
                    case ExchangeConfigConsts::modeDeleteServer:
                        return "#Delete_Server";
                    default:
                        return "#Servers";
                }
            default: return "";
        }
    }
    
    public function getDefaultTab() {
        return ExchangeConfigConsts::tabServer;
    }
    
    public function getHead() {
        return "";//($this->parent->filterId == 0 ? "" : htmlentities_utf8($this->parent->filterName));
    }
    
    public function createFields() {
        $this->addSection(ExchangeConfigConsts::tabPgsql);
        $this->addSection(ExchangeConfigConsts::tabServer);
        $genericHint = "";
        
        $this->parent->addTabMenuItem(tl::tl("Server"), "javascript:submitTab('".ExchangeConfigConsts::tabServer."')", $this->parent->currentTab == ExchangeConfigConsts::tabServer);
        $this->parent->addTabMenuItem(tl::tl("Database"), "javascript:submitTab('".ExchangeConfigConsts::tabPgsql."')", $this->parent->currentTab == ExchangeConfigConsts::tabPgsql);
        
        switch($this->parent->currentTab) {
            case ExchangeConfigConsts::tabPgsql:
                $this->tabPgSql("innovaphone-exchange", ExchangeConfigConsts::pageConfig);
                break;
            case ExchangeConfigConsts::tabServer:
                $this->tabServer();
                break;
        }
    }
    
    function tabServer() {
        global $exchangeSession;
        $this->currentSection = $this->parent->currentTab;
        $this->addActionText();
        
        $error = InputPage::formInput("error");
        if (isset($error)) {
            $errorField = $this->addField(new InputPageText("error", ""));
            $errorField->value = "<b>" . tl::tl("An unexpected error occurred:") . "</b>";
            $errorField->value .= "<br><div style=\"color:red\">$error</div><br><br>";
            $errorField->fullline = true;
        }
        $error = MainMethods::checkRunningScript(Config::configErrorPostgres);
        if($error != "") {
            $running = $this->addField(new InputPageText("procrunning", $error));
            $running->fullline = true;
            return;
        }
        $createName = tl::tl("Add");
        $create = $this->addField(new InputPageText("createserverlink", $this->getModeLink(ExchangeConfigConsts::modeServer, tl::tl("Add"), "", "javascript:submitServer('0', '".ExchangeConfigConsts::modeServer."')")));
        $create->fullline = true;
        switch($this->parent->currentMode) {
            case ExchangeConfigConsts::modeServer:
                $this->modeServer();
                break;
            case ExchangeConfigConsts::modePbx:
                $this->modePbx();
                break;
            case ExchangeConfigConsts::modeEws:
                $this->modeEws();
                break;
            case ExchangeConfigConsts::modeDeleteServer:
                $this->modeDeleteServer();
                break;
        }
        if($this->parent->currentMode == "") {
            $this->addServerList();
        }
    }
    
    function addServerList() {
        // fields below the green field, here the filter list
        $this->addSection(ExchangeConfigConsts::tabServer."_bottom", 1, false);
        $this->currentSection = ExchangeConfigConsts::tabServer."_bottom";
        
        $list = $this->addField(new MainInputPageListField("listserver"));
        $emptyText = tl::tl("No servers yet");
        $empty = $list->fieldOnEmptyTable = new InputPageText($list->name."emptytable", 
            "<b>$emptyText</b>");
        $empty->style = $this->currentSection;
        $list->addColumn($name = new InputPageText($list->name."name", null), tl::tl("Server"));
        $name->sortType = genericInputPageItem::sortAlphanumericCaseIndependent;
        $list->addColumn($desc = new InputPageText($list->name."desc", null), tl::tl("Description"));
        $desc->sortType = genericInputPageItem::sortAlphanumericCaseIndependent;
        $list->addColumn($last = new MainInputPageNoBoxDateField($list->name."update", null, 0, null, true), tl::tl("Last Update"));
        $list->addColumn($id = new InputPageHiddenIdField($list->name."id"));
    }
    
    function modeDeleteServer() {
        $this->currentSection = ExchangeConfigConsts::modeDeleteServer;
        $this->addSection($this->currentSection, 1, true, tl::tlx('Delete the selected server \'{$arg[1]}\'?', htmlentities_utf8($this->parent->serverName)));
        $this->addField($del = new ButtonDeleteServer("buttondeleteserver", ExchangeConfigConsts::pageConfig, $this->parent, "", ExchangeConfigConsts::modeServer));
    }
    
    function modeServer() {
        /* @var $exchangeSession ExchangeSession */ global $exchangeSession;
        $this->currentSection = ExchangeConfigConsts::modeServer;
        $this->addSection($this->currentSection, 1, true, tl::tl("Server Configuration"));
        
        $server = $exchangeSession->exServer;
        $this->addField($name = new InputPageStringField("name", tl::tl("Name"), $server->name));
        $this->addField($desc = new InputPageStringField("desc", tl::tl("Description"), $server->desc));
        $desc->emptyOk = true;
        $langs = array();
        foreach (tl::languages() as $shorthand => $path) {
            $langs[$shorthand] = tl::iso639($shorthand);
        }
        $this->addField($lang = new InputPageDropdownField("calendarlang", tl::tl("Calendar Note Language"), $langs));
        $lang->default[$server->lang] = true;
        $this->addField($interval = new PbxPresenceUpdateField("interval", tl::tl("Presence Update Interval"), $server->interval));
        $interval->hint = tl::tl("The interval, in which the calender presence will be updated inside the PBX and PBX user objects are synchronised.");
        
        $this->addField($trace = new InputPageCheckBoxField("trace", tl::tl("Trace")));
        $nameValue = InputPage::formInput("name");
        if(!isset($nameValue)) {
            $trace->default["on"] = $server->trace;
        }
        $this->addField($btn = new ButtonServerConfig("buttonserverconfig", ExchangeConfigConsts::pageConfig, $this->parent, ExchangeConfigConsts::modePbx));
        if($this->parent->serverId > 0) {
            $del = $this->addField(new ButtonAskDeleteServer("buttondeleteserver", tl::tl("Delete")));
        }
        $name->setButtons($btn->name, $btn->name."_cancel");
        $desc->setButtons($btn->name, $btn->name."_cancel");
        $lang->setButtons($btn->name, $btn->name."_cancel");
        $interval->setButtons($btn->name, $btn->name."_cancel");
        $trace->setButtons($btn->name, $btn->name."_cancel");
    }
    
    function modePbx() {
        /* @var $exchangeSession ExchangeSession */ global $exchangeSession;
        $this->currentSection = ExchangeConfigConsts::modePbx;
        $this->addSection($this->currentSection, 1, true, tl::tl("PBX Configuration"));
        
        $server = $exchangeSession->exServer;
        $this->addField($ip = new InputPageIPField("ip", tl::tl("IP Address"), $server->pbx->ip));
        $ip->emptyOk = true;
        $ip->hint = tl::tl("The IP address of your PBX.");
        $this->addField($port = new InputPageDigitstringField("port", tl::tl("Port"), $server->pbx->port));
        $port->emptyOk = true;
        $port->placeHolder = "80";
        $port->hint = tl::tl("The HTTP(s) port of your PBX.");
        $this->addField($account = new InputPageStringField("account", tl::tl("Account"), $server->pbx->account));
        $account->emptyOk = true;
        $account->hint = tl::tl("The account, which will be used for the SOAP connection.");
        $this->addField($pw = new InputPagePasswordField("pw", tl::tl("Password"), $server->pbx->pw));
        $pw->minlength = 1;
        $pw->emptyOk = true;
        $this->addField($dynPbx = new InputPageStringField("dynpbx", "Dyn-PBX-ID", $server->pbx->dynPbx));
        $dynPbx->emptyOk = true;
        $dynPbx->hint = tl::tl("Enter the ID of the dynamic PBX, if you use one.");
        $accountValue = InputPage::formInput("account");
        $this->addField($ssl = new InputPageCheckBoxField("ssl", tl::tl("Use HTTPS")));
        if(!isset($accountValue)) {
            $ssl->default["on"] = $server->pbx->ssl;
        }
        $this->addField($test = new ButtonTestPbx("buttontestpbx", tl::tl("Connection Test")));
        $this->addField($btn = new ButtonPbxConfig("buttonpbxconfig", ExchangeConfigConsts::pageConfig, $this->parent, ExchangeConfigConsts::modeEws, ExchangeConfigConsts::modeServer));
        
        $ip->setButtons($btn->name, $btn->name."_cancel");
        $account->setButtons($btn->name, $btn->name."_cancel");
        $pw->setButtons($btn->name, $btn->name."_cancel");
        $dynPbx->setButtons($btn->name, $btn->name."_cancel");
        $ssl->setButtons($btn->name, $btn->name."_cancel");
        $port->setButtons($btn->name, $btn->name."_cancel");
    }
    
    function modeEws() {
        /* @var $exchangeSession ExchangeSession */ global $exchangeSession;
        $this->currentSection = ExchangeConfigConsts::modeEws;
        $this->addSection($this->currentSection, 1, true, tl::tl("Exchange Configuration"));
        
        $server = $exchangeSession->exServer;
        $ewsServer = $this->addField(new InputPageStringField("ewsserver", tl::tl("Server"), $server->ews->server));
        $ewsServer->emptyOk = true;
        $ewsServer->hint = tl::tl("The Microsoft Exchange Server with running Exchange Web Services.");
        $account = $this->addField(new InputPageStringField("ewsaccount", tl::tl("Account"), $server->ews->account));
        $account->emptyOk = true;
        $account->hint = tl::tl("The Microsoft Exchange Account, which has read permission for the calendars of the PBX users.");
        $pw = $this->addField(new InputPagePasswordField("ewspw", tl::tl("Password"), $server->ews->pw));
        $pw->minlength = 1;
        $pw->emptyOk = true;
        $statusFrequency = $this->addField( new ExchangeStatusFrequencyField("statusfrequency", tl::tl("Status Frequency"), $server->ews->statusFrequency));
        $statusFrequency->hint = tl::tl("The status frequency defines the frequency, specified in minutes, at which notification messages will be sent when no events have occurred.");
        $availabilityTimespan = $this->addField( new ExchangeAvailabilityTimespanField("availabilitytimespan", tl::tl("Availability Timespan"), $server->ews->availabilityTimespan));
        $availabilityTimespan->hint = tl::tl("Calendar entries are retrieved for this timespan (hours).");
        $hint = $this->addField(new InputPageText("tentativehint", tl::tl("Tentative calendar entries are shown, if the organizer of the calendar entry is one of the following email addresses:")));
        $hint->attributes["style"] = "margin-top:10px;max-width:550px";
        $tentativeMails = $this->addField(new InputPageMultiMailField("tentativemails", tl::tl("Email(s)"), $server->ews->tentativeMails));
        $tentativeMails->hint = tl::tl("Enter a comma separated list of primary SMTP email addresses.");
        $tentativeMails->emptyOk = true;
        $versions = array(EWS_RequestServerVersion::Version_Exchange2010_SP1 => "2010 SP1",
                        EWS_RequestServerVersion::Version_Exchange2010_SP2 => "2010 SP2",
                        EWS_RequestServerVersion::Version_Exchange2013 => "2013");
        $version = $this->addField(new InputPageDropdownField("version", tl::tl("Exchange Version"), $versions));
        $version->default[$server->ews->version] = true;
        $this->addField($test = new ButtonTestEws("buttontestews", tl::tl("Connection Test")));
        
        $this->addField($linuxText = new InputPageText("linuxtxt", "<br>".tl::tl("If your Linux AP is behind a NAT router in the view of your Exchange server, you have to configure a reachable Linux IP/Linux Port.")));
        $linuxText->value .= "<br>".tl::tl("This IP address has to be reachable from inside your Exchange server!");
        $this->addField($linuxIp = new InputPageIPField("linuxip", "Linux NAT IP", $server->ews->linuxIp));
        $linuxIp->emptyOk = true;
        $this->addField($linuxPort = new InputPageDigitstringField("linuxport", "Linux NAT Port", $server->ews->linuxPort));
        $linuxPort->emptyOk = true;
        
        $this->addField($btn = new ButtonExchangeConfig("buttonewsconfig", ExchangeConfigConsts::pageConfig, $this->parent, "", ExchangeConfigConsts::modePbx));
        
        $ewsServer->setButtons($btn->name, $btn->name."_cancel");
        $account->setButtons($btn->name, $btn->name."_cancel");
        $pw->setButtons($btn->name, $btn->name."_cancel");
        $statusFrequency->setButtons($btn->name, $btn->name."_cancel");
        $availabilityTimespan->setButtons($btn->name, $btn->name."_cancel");
        $tentativeMails->setButtons($btn->name, $btn->name."_cancel");
        $version->setButtons($btn->name, $btn->name."_cancel");
        $linuxIp->setButtons($btn->name, $btn->name."_cancel");
        $linuxPort->setButtons($btn->name, $btn->name."_cancel");
    }
    
    function addActionText() {
        $text = $this->addField(new InputPageText("filtertext_{$this->currentSection}" , ""));
        $text->autorender = false;
        $text->fullline = true;
        $createdServer = InputPage::formInput("createdserver");
        $editedServer = InputPage::formInput("editedserver");
        $deletedServer = InputPage::formInput("deletedserver");
        
        if(isset($createdServer )) {
            $text->value = tl::tlx('Server {$arg[1]} successfully created.', "<b>".htmlentities_utf8($createdServer)."</b>");
        }
        elseif(isset($editedServer )) {
            $text->value = tl::tlx('Server {$arg[1]} successfully edited.', "<b>".htmlentities_utf8($editedServer)."</b>");
        }
        elseif(isset($deletedServer)) {
            $text->value = tl::tlx('Server {$arg[1]} successfully deleted.', "<b>".htmlentities_utf8($deletedServer)."</b>");
        }
        
        if($text->value != "") {
            $text->autorender = true;
            $text->value = "<i>{$text->value}</i><p>&nbsp;</p>";
        }
    }
    
    function fillServerList() {
        $list = $this->parent->getField("listserver");
        if(!is_object($list)) return;
        $rows = 0;
        
        $exchangeDb = get_exchange_db();
        $query = "SELECT id, name, description, last_update FROM servers ORDER BY LOWER(name)";
        $result = $exchangeDb->run($query);
        while(($row = pg_fetch_object($result))) {
            $list->value[$rows][$list->name."name"] = "<a id=\"".htmlentities_utf8($row->name)."\" class=\"myinno-listlink\" href=\"javascript:submitServer('{$row->id}', '".ExchangeConfigConsts::modeServer."')\">".htmlentities_utf8($row->name)."</a>";
            $list->value[$rows][$list->name."desc"] = $row->description;
            $list->value[$rows][$list->name."id"] = $row->id;
            $list->value[$rows][$list->name."update"] = (isset($row->last_update) ? strtotime($row->last_update) : tl::tl("N/A"));
            $rows++;
        }
    }
    
    public function fillLists() {
        switch($this->parent->currentTab) {
            case ExchangeConfigConsts::tabServer:
                switch($this->parent->currentMode) {
                    case "":
                        $this->fillServerList();
                        break;
                }
                break;
            case ExchangeConfigConsts::tabPgsql:
                $this->fillIpList("innovaphone-exchange");
                break;
        }
    }
}

final class ExchangeConfig extends ExchangeInputPage {
    var $serverId = 0;
    var $serverName = "";
    
    // constructor
    public function ExchangeConfig() {
        parent::ExchangeInputPage(tl::tl("Exchange Calendar Connector Config"), "exchangeconfig");
        /* @var $exchangeSession ExchangeSession */ global $exchangeSession;
        $this->setFieldSectionAttributes("inline", new _InputPageFieldSectionAttributes(array(), 1));
        
        // dummy fields to save certain parameters over all pages
        $this->addField(new InputPageHiddenIdField("server_id", $this->serverId));
        $this->serverId = InputPage::formInput("server_id");
        if(!isset($this->serverId)) {
            $this->serverId = 0;
        }
        else {
            $this->serverId = intval($this->serverId);
        }
        
        switch($this->currentMode) {
            case "":
                $exchangeSession->updateExchangeServer(new ExchangeServer());
                break;
            case ExchangeConfigConsts::modeServer:
            case ExchangeConfigConsts::modePbx:
            case ExchangeConfigConsts::modeEws:
            case ExchangeConfigConsts::modeDeleteServer:
                if($exchangeSession->exServer->id != $this->serverId) {
                    if($this->serverId > 0) {
                        $server = new ExchangeServer();
                        $server->read($this->serverId);
                        if($server->id > 0) {
                            $exchangeSession->updateExchangeServer($server);
                        }
                        else {
                            $exchangeSession->updateExchangeServer(new ExchangeServer());
                            $this->serverId = 0;
                        }
                    }
                    else {
                        $exchangeSession->updateExchangeServer(new ExchangeServer());
                    }
                }
                break;
        }
        $this->serverName = $exchangeSession->exServer->name;
        
        // add proper fields to pages
        $this->pages = array(ExchangeConfigConsts::pageConfig => new PageConfig($this, false, tl::tl("Exchange Calendar Connector Config")));
        
        // update page
        $this->setDefaultTabProperties();
        $this->pages[$this->pageNo]->createFields();
    }
    
    public function getArgs() {
        if($this->serverId > 0) {
            $id = "server_id={$this->serverId}&";
        }
        return "{$id}tab={$this->currentTab}";
    }
};

// create page
$page = new ExchangeConfig;

if($page->pageNo == ExchangeConfigConsts::pageConfig) {
    $page->updateFields();
    $page->pages[$page->pageNo]->fillLists();
}

$page->work();

close_exchange_db();
?>
