#!/bin/sh
# Install postgresql-9.1
#
#
# Some checkings before installing

trap "on_error; on_exit; exit" INT QUIT TERM

installed_packages=""
cfg_tmp_dir=""
config_pg_dir=""
config_pg_file=""
innovaphone_user=""
innovaphone_group=""
database_user=""
database_name=""
exchange_tmp=""
cron_table=""
username=""
groupname=""

function on_exit() {
    if [ -n "$cfg_tmp_dir" ]; then
        rm -rf $cfg_tmp_dir
    fi
    if [ -n "$exchange_tmp" ]; then
        rm -rf $exchange_tmp
    fi
    apt-get update
    apt-get clean
} 

function on_error() {
    # cron
    if [ -n "$cron_table" ]; then
	if [ -f "$cron_table" ]; then
            rm $cron_table
	fi
    fi
    # drop database
    if [ -n "$database_name" ]; then
        sudo -u postgres dropdb $database_name
    fi
    # drop database user
    if [ -n "$database_user" ]; then
        sudo -u postgres psql -c "DROP USER ${database_user}" postgres 2>/dev/null
    fi
    # uninstall packages
    if [ -n "$installed_packages" ]; then
        apt-get -y --force-yes purge $installed_packages 
	apt-get autoremove
    fi
    # linux-user
    if [ -n "$innovaphone_user" ]; then
        deluser $innovaphone_user 2>/dev/null
    fi
    if [ -n "$innovaphone_group" ]; then
        delgroup $innovaphone_group 2>/dev/null
    fi
    if [ -n "$cfg_tmp_dir" -a -n "$config_pg_dir" ]; then
        cp $cfg_tmp_dir/postgresql.conf $config_pg_dir 2>/dev/null
        cp $cfg_tmp_dir/pg_hba.conf $config_pg_dir 2>/dev/null
    fi
    on_exit
}

version=`tr -d '\n\r ' < version`
hotfix=`tr -d '\n\r ' < hotfix`
echo "LOG: $(date '+%Y-%m-%d %H:%M:%S') - Installing innovaphone-exchange build ${version} hotfix${hotfix}"

chmod 755 check_platform.sh
./check_platform.sh platform
if [ $? -ne 0 ]; then
    exit 1
fi

if [ -f "/var/www/innovaphone/apps/innovaphone-exchange.xml" ]; then
    echo "ERROR: $(date '+%Y-%m-%d %H:%M:%S') - innovaphone-exchange is already installed. Install an incremental or cumulative hotfix instead!"
    exit 1
fi

image_hotfix=0
if [ -f "/var/www/innovaphone/hotfix" ]; then
    image_hotfix=`tr -d '\n\r ' < /var/www/innovaphone/hotfix`
fi
hotfix_offset=0
if [ -f "hotfix_offset" ]; then
    hotfix_offset=`cat hotfix_offset`
fi
if [ $((${hotfix} + ${hotfix_offset})) -gt ${image_hotfix} ]; then
    echo "ERROR: $(date '+%Y-%m-%d %H:%M:%S') - Install linux application platform hotfix${hotfix} first to install innovaphone-exchange hotfix${hotfix}!"
    exit 1
elif [ $((${hotfix} + ${hotfix_offset})) -lt ${image_hotfix} ]; then
    echo "ERROR: $(date '+%Y-%m-%d %H:%M:%S') - Install innovaphone-exchange hotfix$((${hotfix} + ${hotfix_offset})), as linux application hotfix${image_hotfix} is already installed!"
    exit 1
fi

if [[ $EUID -ne 0 ]]; then
    echo "LOG: $(date '+%Y-%m-%d %H:%M:%S') - This script must be run as root"
    exit 1
fi

if [ -z "$(ps auxwww | grep lighttpd | grep -v grep)" ] ; then
    echo "ERROR: $(date '+%Y-%m-%d %H:%M:%S') - LIGHTTPD not running"
    exit 1
fi

echo "LOG: $(date '+%Y-%m-%d %H:%M:%S') - Check if everything is ok with Lighttpd web server"
lighttpd_conf=`cat /etc/init.d/lighttpd | egrep -o '(/[^\/]+)+/lighttpd.conf'`
if [ ${#lighttpd_conf} -eq "0" -o ! -f $lighttpd_conf ]; then
    if [ -e "/etc/lighttpd/lighttpd.conf" ]; then
        lighttpd_conf="/etc/lighttpd/lighttpd.conf"
    else
        echo "ERROR: $(date '+%Y-%m-%d %H:%M:%S') - Lighttpd config file not found"
        exit 1
    fi
fi

doc_roots=( $(cat $lighttpd_conf | tr "\011" " " | egrep -oe "^[ ]*server.document-root[ ]*=[ ]*"".*$""" | cut -d"=" -f2 | sed -e "s/^[^/]*//" | grep -oe '^[^"]*') )
num_docs=${#doc_roots[@]}
last=$(($num_docs-1))
doc_root=${doc_roots[$last]}
groupname=`cat $lighttpd_conf | tr "\011" " " | egrep -oe "^[ ]*server.groupname[ ]*=[ ]*"".*$""" | cut -d"=" -f2 | sed -e 's/^[^"]*"//' -e 's/"*$//'`
username=`cat $lighttpd_conf | tr "\011" " " | egrep -oe "^[ ]*server.username[ ]*=[ ]*"".*$""" | cut -d"=" -f2 | sed -e 's/^[^"]*"//' -e 's/"*$//'`
if [ -z "$doc_root" -o -z "$groupname" -o -z "$username" ]; then
    echo "E: document root, groupname or username were not found: $doc_root - $groupname - $username"
    exit 1
fi

test0="^[ ]*server.document-root[ ]*=[ ]*\\\""
test0=$test0$doc_root"\\\""
r_test0=`cat $lighttpd_conf | tr "\011" " " | egrep -oe "$test0"`
test1="^[ ]*server.groupname[ ]*=[ ]*\\\""
test1=$test1$groupname"\\\""
r_test1=`cat $lighttpd_conf | tr "\011" " " | egrep -oe "$test1"`
test2="^[ ]*server.username[ ]*=[ ]*\\\""
test2=$test2$username"\\\""
r_test2=`cat $lighttpd_conf | tr "\011" " " | egrep -oe "$test2"`

if [ -z "$r_test0" -o  -z "$r_test1" -o  -z "$r_test2" ]; then
    echo "E: document root, groupname or username were not found: $doc_root - $groupname - $username"
    exit 1
fi

echo "Document-Root = $doc_root"
echo "Groupname = $groupname"
echo "Username = $username"

if [ ! -d /tmp ]; then
    echo "LOG: $(date '+%Y-%m-%d %H:%M:%S') - Directory tmp does not exists, create"
    mkdir /tmp
    chmod 777 /tmp
    chmod a+t /tmp    
fi

# Keep the config files that will be modified in case something goes wrong...
cfg_tmp_dir=`mktemp -d /tmp/config.$$.XXXX`
if [ $? -ne 0 ]; then
    echo "ERROR: $(date '+%Y-%m-%d %H:%M:%S') - Can't create temp config file, exiting..."
    on_error
    exit 1
fi
chmod 777 $cfg_tmp_dir

deb_p="deb_packages.tar.gz"
if [ -e $deb_p ]; then
    echo "LOG: $(date '+%Y-%m-%d %H:%M:%S') - Untar debian packages"
    deb_f=`cat /etc/apt/sources.list | tr "\011" " " | grep -oe "^[ ]*deb file:///var/cache/apt/archives /"`
    if [ -z "$deb_f" ]; then
        echo "deb file:///var/cache/apt/archives /" >> /etc/apt/sources.list
    fi
    tar xfz $deb_p -C /var/cache/apt/archives
    chown -R root:root /var/cache/apt/archives
    chmod 644 /var/cache/apt/archives/*
    cp /etc/apt/sources.list /etc/apt/sources.list.backup
    echo 'deb file:///var/cache/apt/archives /' > /etc/apt/sources.list
    echo "LOG: $(date '+%Y-%m-%d %H:%M:%S') - Install delivered packages"
    apt-get update
    cp /etc/apt/sources.list.backup /etc/apt/sources.list
else
    echo "ERROR: $(date '+%Y-%m-%d %H:%M:%S') - No debian packages available in installation script"
    exit 1
fi

# Install all php packages needed for PHP
if [ -z "$(dpkg -l | grep '^ii[ ]*php5-mcrypt')" ] ; then
    echo "LOG: $(date '+%Y-%m-%d %H:%M:%S') - Install php5-mcrypt"
    apt-get -y --force-yes install php5-mcrypt
    installed_packages=$installed_packages" php5-mcrypt" 
fi

pg_installed=0
if [ -f "/etc/postgresql/9.1/main/pg_hba.conf" ]; then
    pg_installed=1
fi
echo "LOG: $(date '+%Y-%m-%d %H:%M:%S') - Install POSTGRESQL 9.1"
if [ -n "$(dpkg -l | grep '^ii[ ]*postgresql-9.1')" ] ; then
    echo "LOG: $(date '+%Y-%m-%d %H:%M:%S') - Postgresql already installed"
    if [ $(pg_lsclusters | wc -l) -eq "1" ]; then
        echo "LOG: $(date '+%Y-%m-%d %H:%M:%S') - No cluster found, create a new one"

        postgresql_install=`pg_createcluster -u postgres --start 9.1 main -e utf8 --locale=en_GB.UTF-8 --lc-collate=C`

        config_pg_dir=`echo $postgresql_install | tr "\011" " " | egrep -io "configuration[ ]*:[ ]*(/[^/, ]+)*" | cut -d" " -f2`
        data_pg_dir=`echo $postgresql_install | tr "\011" " " | egrep -io "data[ ]*:[ ]*(/[^/, )]+)*" | cut -d" " -f2`
        config_pg_file=`echo $postgresql_install | egrep -o "Configuring[ ]+[^ ]+[ ]+to[ ]+use" | cut -d" " -f2`
        port_number=`echo $postgresql_install | egrep -o "Configuring[ ]+[^ ]+[ ]+to[ ]+use[ ]+port[ ]+[0-9]{4,5}" | cut -d" " -f6`
        cluster=`echo $postgresql_install | egrep -oe "Starting PostgreSQL 9.1.*$" | cut -d" " -f6 | sed -e 's/[.]*$//'`    
    else
        version=`pg_lsclusters | egrep -oe "^9\.[0-9]+"`
        if [ -z "$version" ]; then
             echo "LOG: $(date '+%Y-%m-%d %H:%M:%S') - Version could not be found. Using 9.1"
             version="9.1"
        fi
        cluster=`pg_lsclusters | tr "\011" " " | egrep -oe "^9\.[0-9][ ]*[^ ]+" | sed -e "s/^[0-9\. ]\+//"`
        if [ -z "$cluster" ]; then
             echo "LOG: $(date '+%Y-%m-%d %H:%M:%S') - Cluster name could not be found. Use main"
             cluster="main"
        fi
        config_pg_file="postgresql.conf"
        config_pg_dir=/etc/postgresql/$version/$cluster/
        data_pg_dir=/var/lib/postgresql/$version/$cluster/
        port_number=5432	# config
    fi
else
    echo "ERROR: $(date '+%Y-%m-%d %H:%M:%S') - Postgresql not installed!"
    exit 1
fi

echo "LOG: $(date '+%Y-%m-%d %H:%M:%S') - CONFIG DIR: $config_pg_dir"
echo "LOG: $(date '+%Y-%m-%d %H:%M:%S') - CONFIGURATION: $config_pg_file"
echo "LOG: $(date '+%Y-%m-%d %H:%M:%S') - DATA DIR: $data_pg_dir"
echo "LOG: $(date '+%Y-%m-%d %H:%M:%S') - PORT NUMBER: $port_number"
echo "LOG: $(date '+%Y-%m-%d %H:%M:%S') - POSTGRESQL CLUSTER: $cluster"

#
# Get here config file location from postgresql_install	
#
pgsql_cfg="${config_pg_dir}/${config_pg_file}"
cp $pgsql_cfg $cfg_tmp_dir
cp "${config_pg_dir}/pg_hba.conf" $cfg_tmp_dir

postgresql_tmp=`mktemp $cfg_tmp_dir/pg_conf.$$.XXXX`
if [ $? -ne 0 ]; then
    echo "ERROR: $(date '+%Y-%m-%d %H:%M:%S') - Can't create temp config file, exiting..."
    on_error
    exit 1
fi

if [ -e "${config_pg_dir}/pg_hba.conf" ]; then
    cat $config_pg_dir/pg_hba.conf | sed -e '/^[ ]*host.*postgres.*postgres.*127.0.0.1\/32.*md5.*/ d' -e '/^[ ]*host.*innovaphone-exchange.*/ d' > $postgresql_tmp
    # Do this to check actual password through web interface
    echo "host    postgres     postgres         127.0.0.1/32             md5" >> $postgresql_tmp
    echo "host    innovaphone-exchange     postgres         127.0.0.1/32             trust" >> $postgresql_tmp
    echo "host    innovaphone-exchange     innovaphone-exchange         127.0.0.1/32             trust" >> $postgresql_tmp
    cp $postgresql_tmp $config_pg_dir/pg_hba.conf
else
    echo "ERROR: $(date '+%Y-%m-%d %H:%M:%S') - pg_hba.conf file does not exist in $config_pg_dir"
    on_error
    exit 1
fi

# Create innovaphone-exchange user
if ! cat /etc/passwd | grep "innovaphone-exchange" &>/dev/null
then
    if ! useradd -m --groups www-data,postgres innovaphone-exchange 2>/dev/null
    then
        echo "ERROR: $(date '+%Y-%m-%d %H:%M:%S') - innovaphone-exchange could not be created"
        on_error
        exit 1
    else
        if ! usermod -aG innovaphone-exchange $username 2>/dev/null
        then
            echo "ERROR: $(date '+%Y-%m-%d %H:%M:%S') - $username could not be added to group innovaphone-exchange"
            on_error
            exit 1
        fi
    fi
    echo "LOG: $(date '+%Y-%m-%d %H:%M:%S') - linux innovaphone-exchange user created"
    innovaphone_user="innovaphone-exchange"
    innovaphone_group="innovaphone-exchange"
fi
if [ ! -d "/home/innovaphone-exchange/tmp" ]; then
    mkdir /home/innovaphone-exchange/tmp
    chmod 777 /home/innovaphone-exchange/tmp
    chown $innovaphone_user:$innovaphone_group /home/innovaphone-exchange/tmp
fi

exchange_conf=/home/innovaphone-exchange/exchange.conf
touch $exchange_conf
echo '<?xml version="1.0" encoding="utf-8"?>' > $exchange_conf
echo '<config>' >> $exchange_conf
echo '</config>' >> $exchange_conf
chmod 770 $exchange_conf
chown innovaphone-exchange:innovaphone-exchange $exchange_conf

# User in postgres group can also modify this file
chmod 660 /etc/postgresql/9.1/main/pg_hba.conf
chmod 775 exchange_database.dump
if [ -z "$(sudo -u postgres psql -c 'select usename from pg_user;' | grep 'innovaphone-exchange')" ]; then
    createUser=`sudo -u postgres psql -c "CREATE USER \"innovaphone-exchange\" WITH PASSWORD 'exchange'" 2>/dev/null`
    if ! echo $createUser | grep "CREATE ROLE"
    then
        echo "ERROR: $(date '+%Y-%m-%d %H:%M:%S') - innovaphone-exchange user could not be created in postgresql"
        on_error
        exit 1
    fi
    echo "LOG: $(date '+%Y-%m-%d %H:%M:%S') - database innovaphone-exchange user created"
    database_user="innovaphone_exchange"
fi

uninstall=/home/innovaphone-exchange/uninstall.sh
mv uninstall.sh $uninstall
chmod 755 $uninstall
chown www-data:www-data $uninstall

mv exchange_database.dump $cfg_tmp_dir
if [ -z "$(su -c 'psql -l' postgres | grep innovaphone-exchange)" ]; then
    sudo -u postgres createdb --encoding=utf8 -T template0 -U postgres innovaphone-exchange
    if [ -z "$(su -c 'psql -l' postgres | grep innovaphone-exchange)" ]; then
        echo "ERROR: $(date '+%Y-%m-%d %H:%M:%S') - innovaphone-exchange database could not be created in postgresql"
        on_error
	exit 1
    fi
    database_name="innovaphone-exchange"
fi
grantUser=`sudo -u postgres psql -c "GRANT ALL PRIVILEGES ON DATABASE \"innovaphone-exchange\" to \"innovaphone-exchange\""`
# sudo -u postgres psql -c "REVOKE ALL PRIVILEGES ON DATABASE postgres FROM \"innovaphone-exchange\""
if ! echo $grantUser | grep "GRANT"
then
    echo "ERROR: $(date '+%Y-%m-%d %H:%M:%S') - rights could not be granted to innovaphone-exchange"
    on_error
    exit 1
fi
sudo -u postgres psql -c "ALTER DATABASE \"innovaphone-exchange\" OWNER TO \"innovaphone-exchange\""
su -c "pg_restore -U innovaphone-exchange -v -s --dbname=innovaphone-exchange $cfg_tmp_dir/exchange_database.dump" innovaphone-exchange
/etc/init.d/postgresql restart
if [ -z "$(ps auxwww | grep postgresql | grep -v 'grep')" ]; then
    echo "LOG: $(date '+%Y-%m-%d %H:%M:%S') - Warning: Postgresql service not running"
fi

if [ ! -f "/home/postgres/restore" ]; then
    mkdir -p /home/postgres/restore
fi    
cp *.psql /home/postgres/restore || echo "LOG: $(date '+%Y-%m-%d %H:%M:%S') - Warning: Sequences will be not updated during backups"

if [ -d "/var/log/postgresql" ]; then
    chmod -R +r /var/log/postgresql
fi

# Install all php packages needed for PHP
echo "LOG: $(date '+%Y-%m-%d %H:%M:%S') - Install exchange files"

innovaphone_dir=$doc_root
innovaphone_apps_dir=$doc_root/apps
exchange_dir=$doc_root/apps/innovaphone-exchange
echo "LOG: $(date '+%Y-%m-%d %H:%M:%S') - innovaphone web server directory: $innovaphone_dir"
echo "LOG: $(date '+%Y-%m-%d %H:%M:%S') - innovaphone exchange directory: $exchange_dir"

# Install exchange cron job
cron_table="/etc/cron.d/innovaphone_exchange"
exchange_script="/var/www/innovaphone/apps/innovaphone-exchange/exchange.php"
exchange_log="/var/www/innovaphone/apps/innovaphone-exchange/log/innovaphone-exchange.log"
if grep $exchange_script < $cron_table 1>/dev/null
then
    echo "$(date '+%Y-%m-%d %H:%M:%S') - exchange script already in crontab"
else
    echo "$(date '+%Y-%m-%d %H:%M:%S') - inserting exchange script to crontab"
    echo "* * * * * root [ -x /usr/bin/php5 ] && nice -1 /usr/bin/php5 -q ${exchange_script} >> ${exchange_log} 2>&1" >> ${cron_table}
fi
/etc/init.d/cron restart

exchange_tar="innovaphone.tar.gz"
if [ -e $exchange_tar ]; then
    exchange_tmp=`mktemp -d ./$(basename $0).XXXXXX`
    if [ $? -ne 0 ]; then
        echo "ERROR: $(date '+%Y-%m-%d %H:%M:%S') - Can't create temp dir for exchange files, exiting..."
	on_error
        exit 1
    fi
    chown $username:$groupname $exchange_tmp
    chmod 777 $exchange_tmp
    chown $username:$groupname ./
    if [ ! -d $innovaphone_dir ]; then
        echo "ERROR: $(date '+%Y-%m-%d %H:%M:%S') - Innovaphone directory does not exist, exiting..."
	on_error
        exit 1
    fi
    if [ ! -d $innovaphone_apps_dir ]; then
        echo "LOG: $(date '+%Y-%m-%d %H:%M:%S') - No application installed, create application directory"
        mkdir $innovaphone_apps_dir
        chown www-data:www-data $innovaphone_apps_dir
    fi

    mv $exchange_tar $exchange_tmp
    cd $exchange_tmp
    if ! tar xfz $exchange_tar
    then
        echo "ERROR: $(date '+%Y-%m-%d %H:%M:%S') - Problem when decompressing files"
        on_error
        exit 1
    fi
    rm $exchange_tar     # remove tar file
    if ! chmod -R 755 * 2>/dev/null
    then
        echo "ERROR: $(date '+%Y-%m-%d %H:%M:%S') - Problem changing permissions"
        on_error
        exit 1
    fi
    
    if ! find . -type f -iname "*.php" | xargs dos2unix -f 2>/dev/null
    then
        echo "ERROR: $(date '+%Y-%m-%d %H:%M:%S') - Problem converting to unix format"
        on_error
        exit 1
    fi

    if ! chown -R $username:$groupname * 2>/dev/null
    then
        echo "ERROR: $(date '+%Y-%m-%d %H:%M:%S') - Problem converting files to lighttpd user and group"
        on_error
        exit 1
    fi
    cd ./apps && chown -R innovaphone-exchange:$groupname ./innovaphone-exchange && chmod -R 775 ./innovaphone-exchange/tmp ./innovaphone-exchange/log && cd ..
    chmod 777 $exchange_script
    cp -pr ./apps/* $innovaphone_apps_dir                                                                # copy exchange files to innovaphone
    cd ./apps/innovaphone-exchange && sudo -u innovaphone-exchange rm -fr * && cd .. && cd ..          # delete all innovaphone-exchange tmp files
    sudo -u $username rm -fr *                 # delete all tmp files
    cd ..

    # Delete temporal files 
    if ! rm -rf $exchange_tmp 2>/dev/null
    then
        echo "LOG: $(date '+%Y-%m-%d %H:%M:%S') - Warning: problem deleting temporal files"
    fi

    dos2unix ${exchange_dir}/version 2>/dev/null
    exchange_version=`cat ${exchange_dir}/version`;
    exchange_hotfix=`cat ${exchange_dir}/hotfix`;
    dos2unix ${exchange_dir}/innovaphone-exchange.xml 2>/dev/null
    echo "    <version>"$exchange_version"</version>" >> ${exchange_dir}/innovaphone-exchange.xml
    echo "    <hotfix>"$exchange_hotfix"</hotfix>" >> ${exchange_dir}/innovaphone-exchange.xml
    if [ -f ${exchange_dir}/hotfix_offset ]; then
        exchange_hotfix_offset=`cat ${exchange_dir}/hotfix_offset`;
        echo "    <hotfix-offset>"$exchange_hotfix_offset"</hotfix-offset>" >> ${exchange_dir}/innovaphone-exchange.xml
    fi
    echo "</application>" >> ${exchange_dir}/innovaphone-exchange.xml
    mv ${exchange_dir}/innovaphone-exchange.xml $innovaphone_dir/apps
    chown $username:$groupname $innovaphone_dir/apps/innovaphone-exchange.xml
    # do not change "Installation was successful", it will be matched!
    echo "LOG: $(date '+%Y-%m-%d %H:%M:%S') - The web server will be restarted a few seconds after the installation!"
    echo "LOG: $(date '+%Y-%m-%d %H:%M:%S') - Installation was successful"
    sleep 10s
    php5-cgi /var/www/innovaphone/update_webserver_cfg.php
else
    echo "ERROR: $(date '+%Y-%m-%d %H:%M:%S') - Exchange files not available"
    on_error
fi

on_exit
exit 1
